part of masamune;

FocusNode useAutoFocusNode([bool autoFocus = true]) {
  final focusNode = useFocusNode();
  useEffect(
    () {
      if (autoFocus) {
        focusNode.requestFocus();
      }
      return () {};
    },
    [focusNode],
  );
  return focusNode;
}

extension WidgetRefFocusNodeExtensions on WidgetRef {
  FocusNode useFocusNode(String key, [bool autoFocus = true]) {
    return containerBuilder<_FocusNodeContainer, FocusNode>(
      key: "focusNode:$key",
      containerBuilder: (container) {
        final focusNode = FocusNode();
        if (autoFocus) {
          focusNode.requestFocus();
        }
        return _FocusNodeContainer(
          focusNode: focusNode,
        );
      },
      resultBuilder: (container) {
        return container.focusNode;
      },
    );
  }
}

@immutable
class _FocusNodeContainer extends VariableContainer {
  _FocusNodeContainer({required this.focusNode});
  final FocusNode focusNode;

  @override
  void didUpdateContainer(_FocusNodeContainer oldContainer) {
    super.didUpdateContainer(oldContainer);
    if (focusNode != oldContainer.focusNode) {
      oldContainer.dispose();
    }
  }

  @override
  void dispose() {
    super.dispose();
    focusNode.dispose();
  }
}
